﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.Web.Management.Client.Win32;

namespace NWSGI.ManagementClient
{
    internal partial class ScriptMappingEditor : TaskForm
    {
        public ScriptMappingInfo Entry { get; private set; }
        public bool HasChanges { get; private set; }

        private bool canAccept;
        protected override bool CanAccept
        {
            get
            {
                if(base.BackgroundJobRunning)
                    return false;

                return canAccept;
            }
        }

        public ScriptMappingEditor(IServiceProvider provider, NwsgiModuleServiceProxy proxy) : this(provider, proxy, null) { }

        public ScriptMappingEditor(IServiceProvider provider, NwsgiModuleServiceProxy proxy, ScriptMappingInfo entry)
            : base(provider)
        {
            this.proxy = proxy;
            
            InitializeComponent();

            this.Entry = entry;
            this.inModificationMode = entry != null;
            if(inModificationMode)
            {
                scriptNameBox.Text = this.Entry.ScriptName;
                scriptNameBox.Enabled = false;

                physicalPathBox.Text = this.Entry.PhysicalPath;
                callableBox.Text = this.Entry.Callable;
            }

            this.UpdateUIState();
            HasChanges = false;
        }

        private void UpdateUIState()
        {
            HasChanges = true;
            canAccept = !string.IsNullOrEmpty(scriptNameBox.Text.Trim());
            base.UpdateTaskForm();
        }

        protected override void OnAccept()
        {
            ScriptMappingInfo item = new ScriptMappingInfo
            { 
                ScriptName = scriptNameBox.Text.Trim(),
                PhysicalPath = physicalPathBox.Text.Trim(),
                Callable = callableBox.Text.Trim()
            };

            base.StartAsyncTask(new DoWorkEventHandler(OnWorkerDoWork), new RunWorkerCompletedEventHandler(OnWorkerCompleted), null, item);
        }

        private void OnWorkerDoWork(object sender, DoWorkEventArgs e)
        {
            ScriptMappingInfo item = (ScriptMappingInfo)e.Argument;

            if(!inModificationMode)
                this.proxy.AddScriptMapping(item);
            else if(HasChanges)
                this.proxy.EditScriptMapping(item);

            this.Entry = item;
        }

        private void OnWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if(e.Error != null)
                this.DisplayErrorMessage(e.Error, Resources.ResourceManager);
            else
            {
                base.Close();
                base.DialogResult = DialogResult.OK;
            }
        }

        private NwsgiModuleServiceProxy proxy;
        private bool inModificationMode;

        private void scriptNameBox_TextChanged(object sender, EventArgs e)
        {
            this.UpdateUIState();
        }

        private void physicalPathBox_TextChanged(object sender, EventArgs e)
        {
            this.UpdateUIState();
        }

        private void callableBox_TextChanged(object sender, EventArgs e)
        {
            this.UpdateUIState();
        }
    }
}
