﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Web;

namespace NWSGI.AspNet
{
    class WsgiAspNetContext : IWsgiContext
    {
        private HttpContext context;
        private IWsgiConfig config;

        private WsgiAspNetRequest request;
        private WsgiAspNetResponse response;
        private ReadOnlyCollection<string> searchPaths;
        
        public WsgiAspNetContext(HttpContext context, IWsgiConfig config)
        {
            this.context = context;
            this.config = config;
        }

        public IWsgiRequest Request
        {
            get { return request ?? (request = new WsgiAspNetRequest(context, wildcard: config.HasWildcard)); }
        }

        public IWsgiResponse Response
        {
            get { return response ?? (response = new WsgiAspNetResponse(context)); }
        }

        public ICollection<string> SearchPaths
        {
            get
            {
                return searchPaths ?? (searchPaths = new ReadOnlyCollection<string>(new []
                {
                    MapPath("~/"),
                    MapPath("~/App_Script"),
                    MapPath("~/bin/Lib")
                }));
            }
        }

        public string MapPath(string path)
        {
            return path.StartsWith("~/") ? context.Server.MapPath(path) : path;
        }
    }
}
