﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System.Collections.Generic;

namespace NWSGI
{
    public interface IWsgiContext
    {
        /// <summary>
        /// The IWsgiRequest object for this request.
        /// </summary>
        IWsgiRequest Request { get; }

        /// <summary>
        /// The IWsgiResponse object for this request.
        /// </summary>
        IWsgiResponse Response { get; }

        /// <summary>
        /// The paths to be added to sys.path.
        /// </summary>
        ICollection<string> SearchPaths { get; }

        /// <summary>
        /// Map a URL path to a filesystem path. If the path is already a filesystem path,
        /// it should be returned unchanged. If the URL starts with ~/, it is relative to
        /// the WSGI application root; what that means may vary from server to server.
        /// </summary>
        /// <param name="path">A URL path or filesystem path.</param>
        /// <returns>The filesystem path corresponding to the given URL.</returns>
        string MapPath(string path);
    }
}
