﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using System.Collections;
using IronPython.Runtime;
using IronPython.Hosting;

namespace NWSGI.Tests
{
    public class EnvironFacts
    {
        private StubConfig config;
        private Microsoft.Scripting.Hosting.ScriptEngine engine;
        
        public EnvironFacts()
        {
            this.config = new StubConfig();

            this.engine = Python.CreateEngine();
        }

        [Fact]
        public void Should_set_REQUEST_METHOD_from_request()
        {
            var wsgi = CreateEnvironCheckExecutor(new StubRequest { Method = "GET" });
            wsgi.Run(CreateEnvironChecker("REQUEST_METHOD", "GET"));
        }

        [Fact]
        public void Should_set_SCRIPT_NAME_from_request()
        {
            var wsgi = CreateEnvironCheckExecutor(new StubRequest { ScriptName = "/hello.wsgi" });
            wsgi.Run(CreateEnvironChecker("SCRIPT_NAME", "/hello.wsgi"));
        }

        [Fact]
        public void Should_set_PATH_INFO_from_request()
        {
            var wsgi = CreateEnvironCheckExecutor(new StubRequest { PathInfo = "/jeff", });
            wsgi.Run(CreateEnvironChecker("PATH_INFO", "/jeff"));
        }

        [Fact]
        public void Should_set_QUERY_STRING_from_request()
        {
            var wsgi = CreateEnvironCheckExecutor(new StubRequest { QueryString = "a=1&b=2", });
            wsgi.Run(CreateEnvironChecker("QUERY_STRING", "a=1&b=2"));
        }

        private static WsgiCallableDelegate CreateEnvironChecker(string envvar, string expected)
        {
            return (environ, start_response) =>
            {
                Assert.Equal(environ[envvar], expected);
                start_response("200 OK", new List(), null);

                return Enumerable.Empty<string>();
            };
        }

        private WsgiExecutor CreateEnvironCheckExecutor(IWsgiRequest request)
        {
            var context = new StubContext
            {
                Request = request,
                Response = new StubResponse()
            };

            return new WsgiExecutor(engine, config, context);
        }
    }
}
