﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using Moq;

namespace NWSGI.Tests
{
    public class ExplicitWriteFacts
    {
        private StubConfig config;
        private Microsoft.Scripting.Hosting.ScriptEngine engine;

        public ExplicitWriteFacts()
        {
            this.config = new StubConfig();

            this.engine = IronPython.Hosting.Python.CreateEngine();
        }

        [Fact]
        public void Should_be_called_with_empty_string_if_no_result_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object);

            response.Verify(r => r.Write(new byte[] { }));
        }

        [Fact]
        public void Should_flush_if_no_result_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object);

            response.Verify(r => r.Flush());
        }

        [Fact]
        public void Should_be_called_once_if_one_result_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object, "Hello, World");

            response.Verify(r => r.Write(It.IsAny<byte[]>()), Times.Once());
        }

        [Fact]
        public void Should_flush_once_if_one_result_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object, "Hello, World");

            response.Verify(r => r.Flush(), Times.Once());
        }

        [Fact]
        public void Should_be_called_multiple_times_if_multiple_results_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object, "Hello", ", ", "World");

            response.Verify(r => r.Write(It.IsAny<byte[]>()), Times.Exactly(3));
        }

        [Fact]
        public void Should_flush_multiple_times_if_multiple_results_yielded()
        {
            var response = new Mock<IWsgiResponse>();

            RunWriteTestApplication(response.Object, "Hello", ", ", "World");

            response.Verify(r => r.Flush(), Times.Exactly(3));
        }

        private void RunWriteTestApplication(IWsgiResponse response, params string[] results)
        {
            var context = new StubContext
            {
                Request = new StubRequest(),
                Response = response
            };

            var wsgi = new WsgiExecutor(engine, config, context);

            wsgi.Run((environ, start_response) =>
            {
                var write = start_response("200 OK", new IronPython.Runtime.List(), null);

                foreach(string result in results)
                {
                    write(result);
                }

                return Enumerable.Empty<string>();
            });
        }
    }
}
