﻿/* **************************************************************************
 *
 * Copyright 2007-2010 Jeff Hardy
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * *************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NWSGI;
using Xunit;
using System.Collections;
using Moq;
using IronPython.Runtime;
using Microsoft.Scripting.Hosting.Providers;

namespace NWSGI.Tests
{
    public class WsgiExecutorTests
    {
        private StubConfig config;
        private Microsoft.Scripting.Hosting.ScriptEngine engine;

        public WsgiExecutorTests()
        {
            this.config = new StubConfig();

            this.engine = IronPython.Hosting.Python.CreateEngine();
        }

        [Fact]
        public void Should_call_close_on_application_result_if_it_exists()
        {
            var application = new Mock<ApplicationWithClose>();

            var context = new StubContext
            {
                Request = new StubRequest(),
                Response = new StubResponse()
            };

            var wsgi = new WsgiExecutor(engine, config, context);

            wsgi.Run((environ, start_response) =>
            {
                start_response("200 OK", new IronPython.Runtime.List(), null);
                return application.Object;
            });

            application.Verify(a => a.close(), Times.Once());
        }

        [Fact]
        public void Should_transmit_file_if_response_is_file_wrapper()
        {
            var response = new Mock<IWsgiResponse>();
            var application = new Mock<ApplicationWithClose>();

            var context = new StubContext
            {
                Request = new StubRequest(),
                Response = response.Object
            };

            var wsgi = new WsgiExecutor(engine, config, context);

            wsgi.Run((environ, start_response) =>
            {
                var file_wrapper = environ["wsgi.file_wrapper"];

                var pyContext = new ModuleContext(new PythonDictionary(), (PythonContext)HostingHelpers.GetLanguageContext(engine));
                var file = new PythonFile(pyContext.GlobalContext);
                file.__init__(pyContext.GlobalContext, "NWSGI.Tests.dll", "r", -1);

                var wrapper = engine.Operations.Invoke(file_wrapper, file);

                return wrapper;
            });

            response.Verify(r => r.TransmitFile(It.IsAny<string>()));
        }
    }
}

public abstract class ApplicationWithClose : IEnumerable
{
    public IEnumerator GetEnumerator()
    {
        return Enumerable.Empty<string>().GetEnumerator();
    }

    public abstract void close();
}
